<?php
namespace tests\Zeedhi\Framework\Zeedhi_Annotations;

use Zeedhi\Framework\Annotations\Parser;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Routing\Router;
use Zeedhi\Framework\DTO\Request;

/**
 * Needed to add bootstrap service's register and to move Annotations
 * tests to end of directory using 'Zeedhi_' as nameprefix, since
 * calling a compile there would break all dependency injection and
 * application mock tests.
 */
class ParserTest extends \PHPUnit\Framework\TestCase {
    /** @var Parser */
    public $parser = null;
    /** @var string */
    public const PATH = './tests/mocks';
    /** @var array */
    public const EXPECTED_OPENAPI = [
        'info' => [
            'title' => 'title',
            'description' => 'description',
            'termsOfUse' => 'Terms of Use',
            'version' => '1.0.0-beta',
            'contact' => [
                'name' => 'Description',
                'url' => 'test',
                'email' => '@teknisa.com',
            ],
            'license' => [
                'name' => 'Description',
                'url' => 'test',
            ],
        ],
        'components' => [
            'securitySchemes' => [
                [
                    'type' => 'oauth2',
                    'flows' => [
                        'implicit' => [
                            'authorizationUrl' => 'https://wso2.teknisa.com/authorization',
                            'tokenUrl' => 'https://wso2.teknisa.com/token',
                            'refreshUrl' => 'https://wso2.teknisa.com/refreshToken',
                            'scopes' => [
                                'write:users' => 'modify data',
                                'read:users' => 'read data',
                            ],
                        ],
                    ],
                ],
            ],
        ],
        'externalDocs' => [
            'description' => 'teknisa external documentation',
            'url' => 'http://teknisa.com',
        ],
        'paths' => [
            'test' => [
                '$ref' => 'referenceMock',
            ],
            'testing' => [
                'summary' => 'summary',
                'description' => 'description',
                'post' => [
                    'summary' => 'summary',
                    'description' => 'description',
                ],
            ],
        ],
        'tags' => [
            [
                'name' => 'test',
                'description' => 'API tag',
            ],
        ],
        'servers' => [
            [
                'url' => 'http://login.teknisa.com/',
                'description' => 'Teknisa login API',
                'variables' => [
                    [
                        'enum' => ['test@teknisa.com',],
                        'default' => 'login@teknisa.com',
                        'description' => 'User login data',
                    ],
                ],
            ],
        ],
        'openapi' => '3.0.0',
    ];

    public function setUp(){
        $instanceManager = InstanceManager::getInstance();
        if(!$instanceManager->isCompiled()) $instanceManager->compile();
        $this->parser = new Parser;
    }

    public function testGetJSON(){
        $openapi = $this->parser::getJSON();
        $this->assertEquals(
            self::EXPECTED_OPENAPI,
            json_decode($openapi, true)
        );
    }

    public function testGetOldJSON(){
        $openapi = $this->parser::getOldJSON(self::PATH);
        $this->assertEquals(self::EXPECTED_OPENAPI, $openapi);
    }

    public function testGetOldJSONwithResource(){
        $openapi = $this->parser::getOldJSON(self::PATH, 'test');
        $this->assertEquals(self::EXPECTED_OPENAPI['paths']['test'], $openapi);
    }

    public function testRegisterControllerRoutes() {
        $ds = DIRECTORY_SEPARATOR;
        $filePath = __DIR__.$ds.'..'.$ds.'ApplicationMocks'.$ds.'controllersRegistry'.$ds.'controllers.json';
        Parser::registerControllerRoutes($filePath);
        $router = new Router();
        $resolvedRoutePost = $router->resolveRoute(new Request(Router::METHOD_POST, '/locations', 'ZhUserTest'));
        $resolvedRouteGet = $router->resolveRoute(new Request(Router::METHOD_GET, '/locations/{LOCATION_ID}', 'ZhUserTest'));
        $this->assertEquals('testing', $resolvedRoutePost[0], 'If the method given is POST, the first position of the retrieved array must be the controller name');
        $this->assertEquals('save', $resolvedRoutePost[1], 'If the method given is POST, the second position of the retrieved array must be the controllerMethod name');
        $this->assertEquals('testing', $resolvedRouteGet[0], 'If the method given is POST, the first position of the retrieved array must be the controller name');
        $this->assertEquals('find', $resolvedRouteGet[1], 'If the method given is POST, the second position of the retrieved array must be the controllerMethod name');
    }
}